/***************************************************************
  *
  *
  *   ShareCom 1.0   
  *    Copyright (C)  1994 Identity Software Inc
  *
  *  srvtest.c
  * 
  *  Futures RPC Test Server
  
  *
  */



#include <stdio.h>

#include <future.h>

#ifndef WINDOWS
#include <memory.h>
/* Can't access DLL state in Windows. */
extern dk_session_t * served_sessions [MAX_SESSIONS];
#endif

#ifdef UNIX
#include <unistd.h>
#endif

#include <stddef.h> 


#ifdef WINDOWS
#define dk_exit(x) \
   PrpcLeave (); \
   exit (x);
#endif

/* local prototypes */
long DK_CALLBACK direct_io_blobi (long length);


/*##**********************************************************************
 * 
 *              fi
 * 
 * Compute the nth term in the Fibonacchi series.  Sample server 
 *  function mapping from long to long.
 *
 * Input params : n - The term to compute
 * 
 * Return value : The nth term.
 * 
 *
 * Limitations   Note the ifndef WINDOWS.  This is a frequently called function
 *                 and a mandatory thread switch at each call would not be 
 *              apptoptiate.  For other operating systems the switch macro
 *             will only switch when the time slice is up and not before, no 
 *                matter how often called.  Windows not having any signal 
 *             mechanism nor native threading this macto cannot be 
 *              used in 'hot' sections of code.
 *
 *
 * Globals used : - none
 */


long DK_CALLBACK fi (long c)
{
#ifndef WINDOWS
  THREAD_ALLOW_SWITCH();
#endif
  return ((c < 2) ? c : (fi (c-1) + fi (c-2)));
}


/*##**********************************************************************
 * 
 *              blobi
 * 
 * Sample server function that increments each byte of a string and returns
 * the string to the caller.  Note the post_blobi function used
 * to free the return value of this.
 *
 * Input params : blob - The DV_LONG_STRING box to be incremented.
 * 
 * Return value : The blob itself.
 * 
 *
 * Limitations -
 *
 * Globals used : - none
 */



caddr_t DK_CALLBACK blobi (caddr_t blob)
{
  int n;
  int length = box_length (blob);
  for (n = 0; n < length; n++) {
    ((char *)blob)[n]++;
    THREAD_ALLOW_SWITCH();
  };
  return(blob);
}


/*##**********************************************************************
 * 
 *              direct_io_blobi
 * 
 * Direct IO version of blobi. This reads the blob from the client insteda 
 * of having the RPC mechanism read it. This makes for multi-threaded reads.
 *  This function threafter writes the incremented blob back and returns
 * the length. Note that the PrpcRedaObject call does not need a length 
 * because the box being read has its own length. The same applies to
 * PrpcWriteObject.
 *
 * Input params : length - ignore
 * 
 * Return value : length - ignore
 * 
 *
 * Limitations -
 *
 * Globals used : - none
 */


long DK_CALLBACK direct_io_blobi (long length)
{
  int ok;
  caddr_t blob;
  dk_session_t* session = IMMEDIATE_CLIENT;
  printf ("\nStarting direct io.\n"); fflush (stdout);
  blob = PrpcReadObject (session);
  printf ("\nRead one< blob.\n");      fflush (stdout);
  PrpcWriteObject (session, blob);
  printf ("\nWrote one blob.\n");     fflush (stdout);
  if (length == box_length(blob))
    ok = length;
  else
    ok = -1;
  dk_free_box(blob);
  return(ok);
}

void DK_CALLBACK post_blobi (caddr_t b, future_request_t * future)
     /* See blobi above */
{
  dk_free_box(b);
}

/*##**********************************************************************
 * 
 *              many_answers
 *
 * Send multiple answers to the client. This constracts a tree of values
 * representing most data types in Level 2 and passes this tree to the client
 * several times before finally returning the last result, a NULL.
 * 
 *
 * Input params : count - The number of partial answerrs to send.
 *               banner - a client-supplied string to be included in each 
 *                            answer.
 * 
 * Return value : BULL
 * 
 *
 * Limitations -
 *
 * Globals used : - none
 */


long DK_CALLBACK many_answers(long count,char * banner)
{
  int i;
  for (i=0; i<count; i++)
    {
      long * longs = (long *) dk_alloc_box (2 * sizeof 
					    (long), DV_ARRAY_OF_LONG);
      float * floats = (float *) dk_alloc_box (2 * sizeof 
					       (float), DV_ARRAY_OF_FLOAT);
      double * doubles = (double *) dk_alloc_box (2 * sizeof 
						  (double), DV_ARRAY_OF_DOUBLE);
      
      caddr_t * arr = (caddr_t *) dk_alloc_box (5*sizeof (caddr_t), 
						DV_ARRAY_OF_POINTER);
      caddr_t * arr2 = (caddr_t *) dk_alloc_box (5 * sizeof (caddr_t), 
						 DV_ARRAY_OF_POINTER);
      
      memset (arr,0, box_length ((caddr_t) arr));                                     
      THREAD_ALLOW_SWITCH();
      
      arr[0] = box_num (100 * i);
      arr[1] = box_num (-20000 * i);
      arr[2] = box_float (1.2 * i);
      arr[3] = box_double ((double)1.2 * i);
      arr[4] = (caddr_t) arr2;
      arr2[0] = box_string (banner);
      /* make a copy of the argument so that the original does not get freed
	 in thecall to dk_free_tree */
      arr2[1] = dk_alloc_box (10, DV_SHORT_STRING);
      /* A binary string of 9 characters. 1 byte reserved for dummy 
	 
	 trailing 0 */
      memcpy (arr2[1], "123456789", 9);
      arr2[1] [9]= 0;
      /* Set the trailing 0 */
      
      arr2 [2] = (caddr_t) longs;
      arr2 [3] = (caddr_t) floats;
      arr2 [4] = (caddr_t) doubles;
      
      PrpcAddAnswer( (caddr_t) arr, DV_ARRAY_OF_POINTER);
      /* wait for five seconds*/
      dk_free_tree (arr);
      fi(17);
    };
  dk_free_box (banner);
  /* Free memory allocated for argument. */
  return (0);
  /* send a 0 to mark the end of the call */
}





/*##**********************************************************************
 * 
 *              report
 *
 * A background 'idle' callback used to report the current status of 
 * this ShareCom server process. 
 * 
 *
 * Input params : 
 * 
 * Return value : 
 * 
 *
 * Limitations - Bot for Windows. The DLL state is not avvessoble. Therefpre 
 *                 just print a constant message.
 *
 * Globals used : - none
 */



#ifdef WINDOWS
void DK_CALLBACK report()
{
  printf ("Background Action.\n");
}
#else

long initbrk;
extern basket_t in_basket;

void report ()
{
  int n;
  int active=0, running=0, served=0;
  int queue;
#ifdef OS2
  while ( 1 )
    {
      active = 0 ;
      running = 0 ;
      served = 0 ;
#endif  
      for (n = 0; n < MAX_THREADS; n++) {
	if (threads[n].thr_IsActive) {
	  active ++;
	  if (threads[n].thr_status == RUNNABLE)
	    running ++;
	};
      };
      for (n = 0; n < MAX_SESSIONS; n++) {
	if (served_sessions[n])
	  served++;
      };
      printf ("Server status: %d Served Sessions, %d threads, %d running.\n",
	      served, active, running);
      {
	s_node_t * token = in_basket.first_token;
	n = 0;
	while (token) {
	  n++;
	  token = token->next;
	};
#ifdef UNIX
	printf ("            %d requests queued.  brk = %d\n",n, sbrk(0)-initbrk);
#else
	printf ("            %d requests queued.  \n",n);
#endif
      };
#ifdef OS2
      DosSleep(5000L ) ;
    }
#endif
}
#endif

/* 
  Remote control exit 
  */
void DK_CALLBACK drop_dead ()
{
  printf ("\n*** Server exiting.\n"); fflush (stdout);
  PrpcDisconnectAll();
  dk_exit(0);
};




#if defined(WINDOWS_APP)

/* If Windows, not Quick Win, application */

#include <windows.h>

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
		   int nCmdShow)
     
#else
     
     void main  (int argc, char ** argv)
     
#endif
{
  char * addr = "1111";
  dk_session_t * listening;

  if (argc > 1)
    addr = argv[1];
  
  
#ifdef WINDOWS
  PrpcSetThreadParams (10000,10000,10000,3);
#endif
  
  PrpcInitialize ();
  
  /* Offer the service */
  PrpcRegisterService ("fi", (server_func) fi,NULL,DV_LONG_INT, NULL);
  /* The name is "fi", the function is fi, it has priority 1
     and returns a long */
  
#ifdef UNIX
  initbrk= (long) sbrk(0);
#endif
  
  
  PrpcRegisterService ("drop_dead", (server_func) drop_dead,NULL,
		       DV_SEND_NO_ANSWER, NULL);
  
  PrpcRegisterService ("binc",(server_func) blobi,NULL,
		       DV_LONG_STRING, post_blobi);
  
  PrpcRegisterService ("binc_one_way", (server_func) blobi,NULL,
		       DV_SEND_NO_ANSWER, post_blobi);
  
  PrpcRegisterService ("many_answers", (server_func) many_answers,NULL,
		       DV_SHORT_INT, NULL);
  
  PrpcRegisterService ("direct_io_binc", (server_func) direct_io_blobi,
		       NULL,DV_LONG_INT, NULL);
  
  
  /* Initialize and start listening */
  listening = PrpcListen (addr, SESCLASS_TCPIP);
  if (! SESSTAT_ISSET (listening->dks_session, SST_LISTENING)) {
    printf ("Failed to start listening");
    dk_exit(1);
  };
  
  /*  PrpcDisconnectAll();
      dk_exit(1);
      */
  
  
  /* Stop the initial thread and leave the scheduler thread running */
  
  
  PrpcSetBackgroundAction ( (background_action_func)  report) ; 
  
  /* Block on the thread's semaphore. Indefinite. */
  semaphore_enter(THREAD_CURRENT_THREAD ->thr_sem);
  
}





